@@ -18,8 +18,9 @@ define([
     'vp_base/js/com/com_util',
     'vp_base/js/com/com_String',
     'vp_base/js/com/component/PopupComponent',
+    'vp_base/js/com/component/SuggestInput',
     'vp_base/js/com/component/MultiSelector'
-], function(bindHtml, bindCss, com_util, com_String, PopupComponent, MultiSelector) {
+], function(bindHtml, bindCss, com_util, com_String, PopupComponent, SuggestInput, MultiSelector) {
 
     /**
      * Bind
@@ -241,8 +242,10 @@ define([
 
                 if (useIndex || that.state.merge.right.useIndex) {
                     $(that.wrapSelector('#vp_bdOn')).attr('disabled', true);
+                    $(that.wrapSelector('#vp_bdLeftOn')).attr('disabled', true);
                 } else {
                     $(that.wrapSelector('#vp_bdOn')).attr('disabled', false);
+                    $(that.wrapSelector('#vp_bdLeftOn')).attr('disabled', false);
                 }
             });
 
@@ -272,8 +275,10 @@ define([
 
                 if (useIndex || that.state.merge.left.useIndex) {
                     $(that.wrapSelector('#vp_bdOn')).attr('disabled', true);
+                    $(that.wrapSelector('#vp_bdRightOn')).attr('disabled', true);
                 } else {
                     $(that.wrapSelector('#vp_bdOn')).attr('disabled', false);
+                    $(that.wrapSelector('#vp_bdRightOn')).attr('disabled', false);
                 }
             });
 
@@ -378,19 +383,31 @@ define([
          * @param {string} defaultValue previous value
          */
         renderVariableList(id, varList, defaultValue='') {
-            var tag = new com_String();
-            tag.appendFormatLine('<select id="{0}">', id);
-            varList.forEach(vObj => {
-                // varName, varType
-                var label = vObj.varName;
-                tag.appendFormatLine('<option value="{0}" data-type="{1}" {2}>{3}</option>'
-                                    , vObj.varName, vObj.varType
-                                    , defaultValue == vObj.varName?'selected':''
-                                    , label);
-            });
-            tag.appendLine('</select>'); // VP_VS_VARIABLES
+            // var tag = new com_String();
+            // tag.appendFormatLine('<select id="{0}">', id);
+            // varList.forEach(vObj => {
+            //     // varName, varType
+            //     var label = vObj.varName;
+            //     tag.appendFormatLine('<option value="{0}" data-type="{1}" {2}>{3}</option>'
+            //                         , vObj.varName, vObj.varType
+            //                         , defaultValue == vObj.varName?'selected':''
+            //                         , label);
+            // });
+            // tag.appendLine('</select>'); // VP_VS_VARIABLES
+            // $(this.wrapSelector('#' + id)).replaceWith(function() {
+            //     return tag.toString();
+            // });
+            let mappedList = varList.map(obj => { return { label: obj.varName, value: obj.varName, dtype: obj.varType } });
+
+            var variableInput = new SuggestInput();
+            variableInput.setComponentID(id);
+            variableInput.addClass('vp-state');
+            variableInput.setPlaceholder('Select variable');
+            variableInput.setSuggestList(function () { return mappedList; });
+            variableInput.setNormalFilter(true);
+            variableInput.setValue(defaultValue);
             $(this.wrapSelector('#' + id)).replaceWith(function() {
-                return tag.toString();
+                return variableInput.toTagString();
             });
         }
 
