@@ -19,8 +19,9 @@ define([
     'vp_base/js/com/com_util',
     'vp_base/js/com/component/PopupComponent',
     'vp_base/js/com/component/SuggestInput',
-    'vp_base/js/com/component/VarSelector'
-], function(frameHtml, frameCss, com_String, com_util, PopupComponent, SuggestInput, VarSelector) {
+    'vp_base/js/com/component/VarSelector',
+    'vp_base/js/m_apps/Subset'
+], function(frameHtml, frameCss, com_String, com_util, PopupComponent, SuggestInput, VarSelector, Subset) {
 
     /**
      * Frame
@@ -35,6 +36,7 @@ define([
                 originObj: '',
                 tempObj: '_vp',
                 returnObj: '_vp',
+                inplace: false,
                 columnList: [],
                 indexList: [],
                 selected: [],
@@ -61,6 +63,10 @@ define([
                 'bool', 'str'
             ];
 
+            // Add/Replace - subset
+            this.subsetCm = null;
+            this.subsetEditor = null;
+
             this.loading = false;
 
             this._addCodemirror('previewCode', this.wrapSelector('#vp_fePreviewCode'), 'readonly');
@@ -100,7 +106,13 @@ define([
                     // initialize state values
                     that.state.originObj = origin;
                     that.state.tempObj = '_vp';
+                    that.state.returnObj = that.state.tempObj;
+                    that.state.inplace = false;
                     that.initState();
+
+                    // reset return obj
+                    $(that.wrapSelector('#vp_feReturn')).val(that.state.tempObj);
+                    $(that.wrapSelector('#inplace')).prop('checked', false);
     
                     // reset table
                     $(that.wrapSelector('.' + VP_FE_TABLE)).replaceWith(function() {
@@ -128,10 +140,33 @@ define([
                 if (returnVariable == '') {
                     returnVariable = that.state.tempObj;
                 }
+                // check if it's same with origin obj
+                if (returnVariable === that.state.originObj) {
+                    $(that.wrapSelector('#inplace')).prop('checked', true);
+                    that.state.inplace = true;
+                } else {
+                    $(that.wrapSelector('#inplace')).prop('checked', false);
+                    that.state.inplace = false;
+                }
+
                 // show preview with new return variable
-                var newCode = that.state.steps[that.state.steps.length - 1];
-                that.setPreview(newCode.replaceAll(that.state.tempObj, returnVariable));
                 that.state.returnObj = returnVariable;
+                that.setPreview(that.getCurrentCode());
+            });
+
+            // check/uncheck inplace
+            $(this.wrapSelector('#inplace')).on('change', function() {
+                let checked = $(this).prop('checked');
+                let returnVariable = '_vp';
+                if (checked === true) {
+                    returnVariable = that.state.originObj;
+                }
+                $(that.wrapSelector('#vp_feReturn')).val(returnVariable);
+
+                // show preview with new return variable
+                that.state.inplace = checked;
+                that.state.returnObj = returnVariable;
+                that.setPreview(that.getCurrentCode());
             });
 
             // menu on column
@@ -478,40 +513,44 @@ define([
         render() {
             super.render();
 
-            this.loadVariableList();
-
             var {
-                originObj,
                 returnObj,
+                inplace,
                 steps
             } = this.state;
+
+            this.loadVariableList();
     
-            $(this.wrapSelector('#vp_feVariable')).val(originObj);
+            $(this.wrapSelector('#vp_feVariable')).val(this.state.originObj);
     
             $(this.wrapSelector('#vp_feReturn')).val(returnObj);
+
+            $(this.wrapSelector('#inplace')).prop('checked', inplace);
     
             // execute all steps
             if (steps && steps.length > 0) {
                 var code = steps.join('\n');
-                this.state.steps = [];
+                // this.state.steps = [];
                 this.loadCode(code);
             }
         }
 
         renderVariableList(varList, defaultValue='') {
-            var tag = new com_String();
-            tag.appendFormatLine('<select id="{0}">', 'vp_feVariable');
-            varList.forEach(vObj => {
-                // varName, varType
-                var label = vObj.varName;
-                tag.appendFormatLine('<option value="{0}" data-type="{1}" {2}>{3}</option>'
-                                    , vObj.varName, vObj.varType
-                                    , defaultValue == vObj.varName?'selected':''
-                                    , label);
+            let mappedList = varList.map(obj => { return { label: obj.varName, value: obj.varName, dtype: obj.varType } });
+
+            var variableInput = new SuggestInput();
+            variableInput.setComponentID('vp_feVariable');
+            variableInput.addClass('vp-state');
+            variableInput.setPlaceholder('Select variable');
+            variableInput.setSuggestList(function () { return mappedList; });
+            variableInput.setSelectEvent(function (value) {
+                $(this.wrapSelector()).val(value);
+                $(this.wrapSelector()).trigger('change');
             });
-            tag.appendLine('</select>'); // VP_VS_VARIABLES
+            variableInput.setNormalFilter(true);
+            variableInput.setValue(defaultValue);
             $(this.wrapSelector('#vp_feVariable')).replaceWith(function() {
-                return tag.toString();
+                return variableInput.toTagString();
             });
         }
 
@@ -530,13 +569,42 @@ define([
             return tag.toString();
         }
 
+        /**
+         * Get last code to set preview
+         * @returns 
+         */
+        getCurrentCode() {
+            let { inplace, steps, tempObj, returnObj } = this.state;
+            let codeList = steps;
+            if (inplace === true) {
+                codeList = steps.slice(1, steps.length);
+            }
+            
+            // get last code
+            let currentCode = codeList[codeList.length - 1];
+            if (currentCode && currentCode != '') {
+                currentCode = currentCode.replaceAll(tempObj, returnObj);
+            } else {
+                currentCode = '';
+            }
+            return currentCode;
+        }
+
         generateCode() {
-            var code = this.state.steps.join('\n');
+            var code = '';
+            // if inplace is true, join steps without .copy()
+            if (this.state.inplace === true) {
+                code = this.state.steps.slice(1).join('\n');
+            } else {
+                code = this.state.steps.join('\n');
+            }
             var returnVariable = $(this.wrapSelector('#vp_feReturn')).val();
             if (returnVariable != '') {
                 code = code.replaceAll(this.state.tempObj, returnVariable);
 
-                code += '\n' + returnVariable;
+                if (code != '') {
+                    code += '\n' + returnVariable;
+                }
             } else {
                 code += '\n' + this.state.tempObj;
             }
@@ -563,8 +631,11 @@ define([
 
         setPreview(previewCodeStr) {
             // get only last line of code
-            var previewCodeLines = previewCodeStr.split('\n');
-            var previewCode = previewCodeLines.pop();
+            var previewCode = previewCodeStr;
+            if (previewCodeStr.includes('\n') === true) {
+                let previewCodeLines = previewCodeStr.split('\n');
+                previewCode = previewCodeLines.pop();
+            }
             this.setCmValue('previewCode', previewCode);
         }
 
@@ -579,6 +650,10 @@ define([
                     // render variable list
                     // get prevvalue
                     var prevValue = that.state.originObj;
+                    if (varList && varList.length > 0 && prevValue == '') {
+                        prevValue = varList[0].varName;
+                        that.state.originObj = prevValue;
+                    }
                     // replace
                     that.renderVariableList(varList, prevValue);
                     $(that.wrapSelector('#vp_feVariable')).trigger('change');
@@ -608,6 +683,9 @@ define([
             content.appendFormatLine('<option value="{0}">{1}</option>', 'value', 'Value');
             content.appendFormatLine('<option value="{0}">{1}</option>', 'calculation', 'Calculation');
             content.appendFormatLine('<option value="{0}">{1}</option>', 'replace', 'Replace');
+            if (type == 'column' || type == 'replace') {
+                content.appendFormatLine('<option value="{0}">{1}</option>', 'subset', 'Subset');
+            }
             content.appendFormatLine('<option value="{0}">{1}</option>', 'apply', 'Apply');
             content.appendLine('</select></td></tr>');
             content.appendLine('</table>');
@@ -654,8 +732,25 @@ define([
             content.appendFormatLine('<div class="{0} {1} {2}" style="display: none;">', 'vp-inner-popup-tab', 'replace', 'vp-scrollbar');
             content.appendLine(this.renderReplacePage());
             content.appendLine('</div>'); // end of vp-inner-popup-tab replace
-            
-            // tab 4. apply
+
+            // tab 4. subset
+            if (type == 'column' || type == 'replace') {
+                content.appendFormatLine('<div class="{0} {1}" style="display: none;">', 'vp-inner-popup-tab', 'subset');
+                content.appendLine('<table class="vp-tbl-gap5"><colgroup><col width="80px"><col width="*"></colgroup>');
+
+                content.appendLine('<tr><td colspan="2">');
+                content.appendLine('<div class="vp-fr-subset-box">');
+                content.appendLine('<textarea class="vp-input vp-inner-popup-subset"></textarea>');
+                content.appendLine('</div>');
+                content.appendLine('</td></tr>');
+
+                content.appendLine('<tr><th><label>Value</label></th>');
+                content.appendFormatLine('<td><input type="text" class="{0}"/>', 'vp-inner-popup-input3');
+                content.appendFormatLine('<label><input type="checkbox" class="{0}" checked/><span>{1}</span></label>', 'vp-inner-popup-istext3','Text');
+                content.appendLine('</td></tr></table>');
+                content.appendLine('</div>'); // end of vp-inner-popup-tab subset
+            }
+            // tab 5. apply
             content.appendFormatLine('<div class="{0} {1}" style="display: none;">', 'vp-inner-popup-tab', 'apply');
             content.appendLine('<table class="vp-tbl-gap5"><colgroup><col width="80px"><col width="*"></colgroup>');
             content.appendLine('<tr><th><label>Column</label></th>');
@@ -666,6 +761,9 @@ define([
             content.appendLine('</tr></table>');
             content.appendLine('</div>'); // end of vp-inner-popup-tab apply
             content.appendLine('</div>'); // end of vp-inner-popup-addpage
+            
+            // set content
+            $(this.wrapSelector('.vp-inner-popup-body')).html(content.toString());
             return content.toString();
         }
 
@@ -693,10 +791,13 @@ define([
             });
             content.appendLine('</table>');
             content.appendLine('</div>');
+
+            // set content
+            $(this.wrapSelector('.vp-inner-popup-body')).html(content.toString());
             return content.toString();
         }
 
-        renderReplacePage = function() {
+        renderReplacePage() {
             var content = new com_String();
             content.appendFormatLine('<label><input type="checkbox" class="{0}"/><span>{1}</span></label>', 'vp-inner-popup-use-regex', 'Use Regular Expression');
             content.appendLine('<br/><br/>');
@@ -709,7 +810,7 @@ define([
             return content.toString();
         }
 
-        renderReplaceInput = function(index) {
+        renderReplaceInput(index) {
             var content = new com_String();
             content.appendLine('<tr>');
             content.appendLine('<td>');
@@ -725,7 +826,7 @@ define([
             return content.toString();
         }
 
-        renderAsType = function() {
+        renderAsType() {
             var astypeList = this.astypeList;
             var content = new com_String();
             content.appendFormatLine('<div class="{0}">', 'vp-inner-popup-astype');
@@ -748,18 +849,62 @@ define([
             });
             content.appendLine('</tbody></table>');
             content.append('</div>');
+
+            // set content
+            $(this.wrapSelector('.vp-inner-popup-body')).html(content.toString());
             return content.toString();
         }
 
-        openInputPopup = function(type, width=400, height=400) {
+        openInputPopup(type, width=400, height=400) {
             var title = '';
             var content = '';
             let size = { width: width, height: height };
+            let that = this;
     
             switch (parseInt(type)) {
                 case FRAME_EDIT_TYPE.ADD_COL:
                     title = 'Add Column';
                     content = this.renderAddPage('column', 'Column Name');
+
+                    // bind codemirror
+                    this.subsetCm = this.initCodemirror({ 
+                        key: 'vp-inner-popup-subset', 
+                        selector: this.wrapSelector('.vp-inner-popup-subset'), 
+                        type: 'readonly' 
+                    });
+                    // set subset
+                    let contentState = that.getPopupContent(type);
+                    this.subsetEditor = new Subset({ 
+                        pandasObject: this.state.tempObj,
+                        selectedColumns: [ com_util.convertToStr(contentState.name, contentState.nameastext) ],
+                        config: { name: 'Subset' } }, 
+                    { 
+                        useInputVariable: true,
+                        useInputColumns: true,
+                        targetSelector: this.wrapSelector('.vp-inner-popup-subset'),
+                        pageThis: this,
+                        allowSubsetTypes: ['iloc', 'loc'],
+                        beforeOpen: function(subsetThis) {
+                            let contentState = that.getPopupContent(type);
+                            let name = com_util.convertToStr(contentState.name, contentState.nameastext);
+                            subsetThis.state.selectedColumns = [ name ];
+                        },
+                        finish: function(code) {
+                            that.subsetCm.setValue(code);
+                            that.subsetCm.save();
+                            setTimeout(function () {
+                                that.subsetCm.refresh();
+                            }, 1);
+                        }
+                    });
+                    // initial code
+                    var code = this.subsetEditor.generateCode();
+                    that.subsetCm.setValue(code);
+                    that.subsetCm.save();
+                    setTimeout(function () {
+                        that.subsetCm.refresh();
+                    }, 1);
+                    
                     break;
                 case FRAME_EDIT_TYPE.ADD_ROW:
                     title = 'Add Row';
@@ -773,6 +918,43 @@ define([
                     title = 'Replace';
                     // content = this.renderReplacePage();
                     content = this.renderAddPage('replace', 'Column');
+
+                    // bind codemirror
+                    this.subsetCm = this.initCodemirror({ 
+                        key: 'vp-inner-popup-subset', 
+                        selector: this.wrapSelector('.vp-inner-popup-subset'), 
+                        type: 'readonly' 
+                    });
+                    // set subset
+                    this.subsetEditor = new Subset({ 
+                        pandasObject: this.state.tempObj,
+                        selectedColumns: that.state.selected.map(col=>col.code),
+                        config: { name: 'Subset' } }, 
+                    { 
+                        useInputVariable: true,
+                        useInputColumns: true,
+                        targetSelector: this.wrapSelector('.vp-inner-popup-subset'),
+                        pageThis: this,
+                        allowSubsetTypes: ['iloc', 'loc'],
+                        beforeOpen: function(subsetThis) {
+                            subsetThis.state.selectedColumns = that.state.selected.map(col=>col.code);
+                        },
+                        finish: function(code) {
+                            that.subsetCm.setValue(code);
+                            that.subsetCm.save();
+                            setTimeout(function () {
+                                that.subsetCm.refresh();
+                            }, 1);
+                        }
+                    });
+                    // initial code
+                    var code = this.subsetEditor.generateCode();
+                    that.subsetCm.setValue(code);
+                    that.subsetCm.save();
+                    setTimeout(function () {
+                        that.subsetCm.refresh();
+                    }, 1);
+
                     break;
                 case FRAME_EDIT_TYPE.AS_TYPE:
                     title = 'Convert type';
@@ -787,15 +969,10 @@ define([
 
             // set size
             $(this.wrapSelector('.vp-inner-popup-box')).css(size);
-
-            // set content
-            $(this.wrapSelector('.vp-inner-popup-body')).html(content);
             
             // bindEventForAddPage
             this.bindEventForPopupPage();
 
-            let that = this;
-
             // set column list
             vpKernel.getColumnList(this.state.tempObj).then(function(resultObj) {
                 let { result } = resultObj;
@@ -863,6 +1040,10 @@ define([
                                 });
                             }
                         }
+                    } else if (tab == 'subset') {
+                        content['subset'] = this.subsetCm?this.subsetCm.getValue():'';
+                        content['value'] = $(this.wrapSelector('.vp-inner-popup-input3')).val();
+                        content['valueastext'] = $(this.wrapSelector('.vp-inner-popup-istext3')).prop('checked');
                     } else if (tab == 'apply') {
                         content['column'] = $(this.wrapSelector('.vp-inner-popup-apply-column')).val();
                         content['apply'] = $(this.wrapSelector('.vp-inner-popup-apply-lambda')).val();
@@ -996,18 +1177,11 @@ define([
                                     });
                                 }
                             } else {
-                                var errorContent = new com_String();
+                                var errorContent = '';
                                 if (msg.content.ename) {
-                                    errorContent.appendFormatLine('<div class="{0}">', VP_FE_INFO_ERROR_BOX);
-                                    errorContent.appendLine('<i class="fa fa-exclamation-triangle"></i>');
-                                    errorContent.appendFormatLine('<label class="{0}">{1}</label>'
-                                                                , VP_FE_INFO_ERROR_BOX_TITLE, msg.content.ename);
-                                    if (msg.content.evalue) {
-                                        // errorContent.appendLine('<br/>');
-                                        errorContent.appendFormatLine('<pre>{0}</pre>', msg.content.evalue.split('\\n').join('<br/>'));
-                                    }
-                                    errorContent.appendLine('</div>');
+                                    errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
                                 }
+                                vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
                                 $(that.wrapSelector('.' + VP_FE_INFO_CONTENT)).replaceWith(function() {
                                     return that.renderInfoPage(errorContent);
                                 });
@@ -1042,15 +1216,15 @@ define([
                     code.appendFormat("{0}.drop([{1}], axis={2}, inplace=True)", tempObj, selectedName, axis);
                     break;
                 case FRAME_EDIT_TYPE.RENAME:
-                    var renameStr = new com_String();
+                    var renameList = [];
                     Object.keys(content).forEach((key, idx) => {
-                        if (idx == 0) {
-                            renameStr.appendFormat("{0}: {1}", content[key].label, com_util.convertToStr(content[key].value, content[key].istext));
-                        } else {
-                            renameStr.appendFormat(", {0}: {1}", content[key].label, com_util.convertToStr(content[key].value, content[key].istext));
+                        if (content[key].value != '') {
+                            renameList.push(com_util.formatString("{0}: {1}", content[key].label, com_util.convertToStr(content[key].value, content[key].istext)));
                         }
                     });
-                    code.appendFormat("{0}.rename({1}={{2}}, inplace=True)", tempObj, axis==FRAME_AXIS.ROW?'index':'columns', renameStr.toString());
+                    if (renameList.length > 0) {
+                        code.appendFormat("{0}.rename({1}={{2}}, inplace=True)", tempObj, axis==FRAME_AXIS.ROW?'index':'columns', renameList.join(', '));
+                    }
                     break;
                 case FRAME_EDIT_TYPE.DROP_NA:
                     var locObj = '';
@@ -1099,12 +1273,12 @@ define([
                     var tab = content.addtype;
                     if (tab == 'value') {
                         var value = com_util.convertToStr(content.value, content.valueastext);
-                        code.appendFormat("{0}[{1}] = {2}", tempObj, name, value);
+                        code.appendFormat("{0}[[{1}]] = {2}", tempObj, name, value);
                     } else if (tab == 'calculation') {
                         var { var1col, oper, var2col } = content;
                         var var1code = tempObj + "['" + var1col + "']";
                         var var2code = tempObj + "['" + var2col + "']";
-                        code.appendFormat('{0}[{1}] = {2} {3} {4}', tempObj, name, var1code, oper, var2code);
+                        code.appendFormat('{0}[[{1}]] = {2} {3} {4}', tempObj, name, var1code, oper, var2code);
                     } else if (tab == 'replace') {
                         var replaceStr = new com_String();
                         var useRegex = content['useregex'];
@@ -1127,8 +1301,11 @@ define([
                             code.append(', regex=True');
                         }
                         code.append(')');
+                    } else if (tab == 'subset') {
+                        var value = com_util.convertToStr(content.value, content.valueastext);
+                        code.appendFormat("{0} = {1}", content.subset, value);
                     } else if (tab == 'apply') {
-                        code.appendFormat("{0}[{1}] = {2}[{3}].apply({4})", tempObj, name, tempObj, content.column, content.apply);
+                        code.appendFormat("{0}[[{1}]] = {2}[{3}].apply({4})", tempObj, name, tempObj, content.column, content.apply);
                     }
                     break;
                 case FRAME_EDIT_TYPE.ADD_ROW: 
@@ -1160,12 +1337,11 @@ define([
             }
     
             var that = this;
-            var tempObj = this.state.tempObj;
-            var lines = this.state.lines;
+            let { tempObj, lines, indexList } = this.state;
             var prevLines = 0;
             var scrollPos = -1;
             if (more) {
-                prevLines = that.state.indexList.length;
+                prevLines = indexList.length;
                 scrollPos = $(this.wrapSelector('.vp-fe-table')).scrollTop();
             }
     
@@ -1327,10 +1503,16 @@ define([
                 // row
                 $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).find('div[data-axis="col"]').hide();
                 $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).find('div[data-axis="row"]').show();
+
+                // change sub-box style
+                $(this.wrapSelector(com_util.formatString('.{0}.vp-fe-sub-cleaning', VP_FE_MENU_SUB_BOX))).css({ 'top': '90px'});
             } else if (this.state.axis == 1) {
                 // column
                 $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).find('div[data-axis="row"]').hide();
                 $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).find('div[data-axis="col"]').show();
+
+                // change sub-box style
+                $(this.wrapSelector(com_util.formatString('.{0}.vp-fe-sub-cleaning', VP_FE_MENU_SUB_BOX))).css({ 'top': '120px'});
             }
             $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).css({ top: top, left: left })
             $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).show();
@@ -1340,6 +1522,21 @@ define([
             $(this.wrapSelector(com_util.formatString('.{0}', VP_FE_MENU_BOX))).hide();
         }
 
+        hide() {
+            super.hide();
+            this.subsetEditor && this.subsetEditor.hide();
+        }
+
+        close() {
+            super.close();
+            this.subsetEditor && this.subsetEditor.close();
+        }
+
+        remove() {
+            super.remove();
+            this.subsetEditor && this.subsetEditor.remove();
+        }
+
     }
 
     const VP_FE_BTN = 'vp-fe-btn';
@@ -1347,6 +1544,7 @@ define([
     const VP_FE_TITLE = 'vp-fe-title';
 
     const VP_FE_MENU_BOX = 'vp-fe-menu-box';
+    const VP_FE_MENU_SUB_BOX = 'vp-fe-menu-sub-box';
     const VP_FE_MENU_ITEM = 'vp-fe-menu-item';
 
     const VP_FE_POPUP_BOX = 'vp-fe-popup-box';
@@ -1363,9 +1561,6 @@ define([
     const VP_FE_INFO = 'vp-fe-info';
     const VP_FE_INFO_CONTENT = 'vp-fe-info-content';
 
-    const VP_FE_INFO_ERROR_BOX = 'vp-fe-info-error-box';
-    const VP_FE_INFO_ERROR_BOX_TITLE = 'vp-fe-info-error-box-title';
-
     const VP_FE_PREVIEW_BOX = 'vp-fe-preview-box';
     const VP_FE_BUTTON_PREVIEW = 'vp-fe-btn-preview';
     const VP_FE_BUTTON_DATAVIEW = 'vp-fe-btn-dataview';
